"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const tls = require("tls");
const net = require("net");
class SocketHelper {
    static createTCPDestinationSocket(hostname, port, sourceSocket, closeFn) {
        return new Promise((resolve, reject) => {
            const destinationSocket = net.connect(port, hostname, (error) => {
                if (error) {
                    reject(error);
                    return;
                }
                destinationSocket.on('data', (buffer) => __awaiter(this, void 0, void 0, function* () {
                    sourceSocket.write(buffer);
                }));
                destinationSocket.on('close', () => {
                    closeFn();
                });
                destinationSocket.on('error', (error) => {
                    if (!error) {
                        return;
                    }
                    closeFn();
                });
                resolve(destinationSocket);
            });
        });
    }
    static createTLSDestinationSocket(hostname, port, sourceSocket, closeFn) {
        return new Promise((resolve, reject) => {
            const destinationSocket = tls.connect(port, {
                host: hostname,
                rejectUnauthorized: false,
            }, () => {
                destinationSocket.on('data', (buffer) => __awaiter(this, void 0, void 0, function* () {
                    sourceSocket.write(buffer);
                }));
                destinationSocket.on('close', () => {
                    closeFn();
                });
                destinationSocket.on('error', (error) => {
                    if (!error) {
                        return;
                    }
                    closeFn();
                });
                resolve(destinationSocket);
            });
        });
    }
}
exports.SocketHelper = SocketHelper;
//# sourceMappingURL=socket.js.map