"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const commander = require("commander");
const child_process_1 = require("child_process");
const fs = require("fs");
const chalk_1 = require("chalk");
const index_1 = require("./index");
commander.version('0.1.2');
commander
    .command('start')
    .option('--serverType <serverType>', 'Server Type, tcp or tls', /^(tcp|tls)$/i, 'tcp')
    .option('--proxyConnectionType <proxyConnectionType>', 'Proxy Connection Type, forward or socks5', /^(forward|socks5)$/i, 'forward')
    .option('--socketType <serverType>', 'Socket Type, tcp or tls', /^(tcp|tls)$/i, 'tcp')
    .option('--hostname <hostname>', 'Hostname', /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/i, '127.0.0.1')
    .option('--port <port>', 'Port', /\d{1,4}/i, 1337)
    .option('--destinationHostname <destinationHostname>', 'Hostname', /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/i, '127.0.0.1')
    .option('--destinationPort <destinationPort>', 'Port', /\d{1,4}/i, 1337)
    .option('--certificatePath <certificatePath>', 'Certificate Path')
    .option('--keyPath <keyPath>', 'Key Path')
    .action(start);
commander
    .command('install')
    .option('--serverType <serverType>', 'Server Type, tcp or tls', /^(tcp|tls)$/i, 'tcp')
    .option('--proxyConnectionType <proxyConnectionType>', 'Proxy Connection Type, forward or socks5', /^(forward|socks5)$/i, 'forward')
    .option('--socketType <serverType>', 'Socket Type, tcp or tls', /^(tcp|tls)$/i, 'tcp')
    .option('--hostname <hostname>', 'Hostname', /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/i, '127.0.0.1')
    .option('--port <port>', 'Port', /\d{1,4}/i, 1337)
    .option('--destinationHostname <destinationHostname>', 'Hostname', /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/i, '127.0.0.1')
    .option('--destinationPort <destinationPort>', 'Port', /\d{1,4}/i, 1337)
    .option('--certificatePath <certificatePath>', 'Certificate Path')
    .option('--keyPath <keyPath>', 'Key Path')
    .action(install);
function install(args) {
    console.log(chalk_1.default.blue(`Creating systemd service file...`));
    fs.writeFileSync('/lib/systemd/system/proxy-server.service', `[Unit]
  Description=Proxy Server
  
  [Service]
  Type=simple
  ExecStart=/usr/bin/proxy-server start --serverType ${args.serverType} --proxyConnectionType ${args.proxyConnectionType} --socketType ${args.socketType} --hostname ${args.hostname} --port ${args.port} --destinationHostname ${args.destinationHostname} --destinationPort ${args.destinationPort} --certificatePath ${args.certificatePath} --keyPath ${args.keyPath}
  
  [Install]
  WantedBy=multi-user.target`);
    console.log(chalk_1.default.blue(`Enabling proxy-server`));
    child_process_1.spawnSync('systemctl', ['enable', 'proxy-server']);
    console.log(chalk_1.default.blue(`Starting proxy-server...`));
    child_process_1.spawnSync('systemctl', ['start', 'proxy-server']);
}
function start(args) {
    let serverBuilder = null;
    switch (args.serverType) {
        case 'tcp':
            serverBuilder = new index_1.TCPServerBuilder();
            break;
        case 'tls':
            serverBuilder = new index_1.TLSServerBuilder().setCertificateFromPath(args.certificatePath).setKeyFromPath(args.keyPath);
            break;
        default:
            throw new Error('Unsupported Server Type');
    }
    let proxyConnectionBuilder = null;
    switch (args.proxyConnectionType) {
        case 'forward':
            proxyConnectionBuilder = new index_1.ForwardProxyConnectionBuilder()
                .setHostname(args.destinationHostname)
                .setPort(parseInt(args.destinationPort));
            break;
        case 'socks5':
            proxyConnectionBuilder = new index_1.SOCKS5ProxyConnectionBuilder();
            break;
        default:
            throw new Error('Unsupported Proxy Connection Type');
    }
    let socketBuilder = null;
    switch (args.socketType) {
        case 'tcp':
            socketBuilder = new index_1.TCPSocketBuilder();
            break;
        case 'tls':
            socketBuilder = new index_1.TLSSocketBuilder();
            break;
        default:
            throw new Error('Unsupported Socket Type');
    }
    const domainEvents = new index_1.DomainEvents();
    const proxyServer = new index_1.ProxyServer(args.hostname, parseInt(args.port), proxyConnectionBuilder, socketBuilder, domainEvents);
    const server = serverBuilder
        .reset()
        .setConnectionFn((socket) => proxyServer.onConnection(socket))
        .build();
    proxyServer.listen(server);
}
commander.parse(process.argv);
//# sourceMappingURL=main.js.map