"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
class ForwardProxyConnection {
    constructor(hostname, port, sourceSocket, socketBuilder) {
        this.hostname = hostname;
        this.port = port;
        this.sourceSocket = sourceSocket;
        this.socketBuilder = socketBuilder;
        this.destinationSocket = null;
        this.initialize();
    }
    close() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.sourceSocket) {
                this.sourceSocket.destroy();
                this.sourceSocket = null;
            }
            if (this.destinationSocket) {
                this.destinationSocket.destroy();
                this.destinationSocket = null;
            }
        });
    }
    onData(buffer) {
        return __awaiter(this, void 0, void 0, function* () {
            while (!this.destinationSocket) {
                yield this.delay(200);
            }
            return this.destinationSocket;
        });
    }
    delay(miliseconds) {
        return new Promise((resolve, reject) => {
            setTimeout(resolve, miliseconds);
        });
    }
    initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            this.destinationSocket = yield this.socketBuilder
                .reset()
                .setHostname(this.hostname)
                .setPort(this.port)
                .setSourceSocket(this.sourceSocket)
                .setCloseFn(() => this.close())
                .build();
        });
    }
}
exports.ForwardProxyConnection = ForwardProxyConnection;
//# sourceMappingURL=forward.js.map