"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const socks5_greeting_request_1 = require("../socks5/socks5-greeting-request");
const socks5_greeting_response_1 = require("../socks5/socks5-greeting-response");
const socks5_connection_request_1 = require("../socks5/socks5-connection-request");
const socks5_connection_response_1 = require("../socks5/socks5-connection-response");
const socks5_authentication_method_1 = require("../socks5/socks5-authentication-method");
const socks5_status_1 = require("../socks5/socks5-status");
const socks5_authentication_request_1 = require("../socks5/socks5-authentication-request");
const socks5_authentication_response_1 = require("../socks5/socks5-authentication-response");
const socks5_authentication_status_1 = require("../socks5/socks5-authentication-status");
class SOCKS5ProxyConnection {
    constructor(sourceSocket, socketBuilder) {
        this.sourceSocket = sourceSocket;
        this.socketBuilder = socketBuilder;
        this.destinationSocket = null;
        this.greetingRequest = null;
        this.greetingResponse = null;
        this.authenticationRequest = null;
        this.authenticationResponse = null;
        this.connectionRequest = null;
        this.connectionResponse = null;
    }
    close() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.sourceSocket) {
                this.sourceSocket.destroy();
                this.sourceSocket = null;
            }
            if (this.destinationSocket) {
                this.destinationSocket.destroy();
                this.destinationSocket = null;
            }
        });
    }
    onData(buffer) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.greetingRequest && !this.greetingResponse) {
                this.handleGreetingRequest(buffer);
                return null;
            }
            if (this.greetingRequest.authenticationMethod !== socks5_authentication_method_1.SOCKS5AuthenticationMethod.NONE &&
                (!this.authenticationRequest && !this.authenticationResponse)) {
                yield this.handleAuthenticationRequest(buffer);
                return null;
            }
            if (!this.connectionRequest && !this.connectionResponse) {
                yield this.handleConnectionRequest(buffer);
                return null;
            }
            if (!this.destinationSocket) {
                this.destinationSocket = yield this.socketBuilder
                    .reset()
                    .setHostname(this.connectionRequest.address)
                    .setPort(this.connectionRequest.port)
                    .setSourceSocket(this.sourceSocket)
                    .setCloseFn(() => this.close())
                    .build();
            }
            return this.destinationSocket;
        });
    }
    handleGreetingRequest(buffer) {
        this.greetingRequest = socks5_greeting_request_1.SOCKS5GreetingRequest.fromBuffer(buffer);
        if (!this.greetingRequest) {
            this.close();
            return;
        }
        const authenticationMethod = null;
        this.greetingResponse = socks5_greeting_response_1.SOCKS5GreetingResponse.fromGreetingRequest(this.greetingRequest, authenticationMethod);
        this.sendResponse(this.greetingResponse);
    }
    handleAuthenticationRequest(buffer) {
        this.authenticationRequest = socks5_authentication_request_1.SOCKS5AuthenticationRequest.fromBuffer(buffer);
        const authenticationStatus = socks5_authentication_status_1.SOCKS5AuthenticationStatus.SUCCESS;
        this.authenticationResponse = socks5_authentication_response_1.SOCKS5AuthenticationResponse.fromAuthenticationRequest(this.authenticationRequest, authenticationStatus);
        this.sendResponse(this.authenticationResponse);
    }
    handleConnectionRequest(buffer) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield socks5_connection_request_1.SOCKS5ConnectionRequest.fromBuffer(buffer);
            if (!result) {
                this.close();
                return;
            }
            this.connectionRequest = result.result;
            this.connectionResponse = socks5_connection_response_1.SOCKS5ConnectionResponse.fromConnectionRequest(this.connectionRequest, result.status);
            this.sendResponse(this.connectionResponse);
            if (result.status !== socks5_status_1.SOCKS5Status.GRANTED) {
                this.close();
                return;
            }
        });
    }
    sendResponse(response) {
        if (this.sourceSocket) {
            this.sourceSocket.write(response.toBuffer());
        }
    }
}
exports.SOCKS5ProxyConnection = SOCKS5ProxyConnection;
//# sourceMappingURL=socks5.js.map