"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
class ProxyServer {
    constructor(hostname, port, proxyConnectionBuilder, socketBuilder, domainEvents) {
        this.hostname = hostname;
        this.port = port;
        this.proxyConnectionBuilder = proxyConnectionBuilder;
        this.socketBuilder = socketBuilder;
        this.domainEvents = domainEvents;
        this.server = null;
    }
    listen(server) {
        this.server = server;
        this.server.listen(this.port, this.hostname, () => {
            this.domainEvents.serverStarted(this.hostname, this.port);
        });
    }
    onConnection(sourceSocket) {
        const proxyConnection = this.proxyConnectionBuilder
            .reset()
            .setSourceSocket(sourceSocket)
            .setSocketBuilder(this.socketBuilder)
            .build();
        sourceSocket.on('data', (buffer) => __awaiter(this, void 0, void 0, function* () {
            this.domainEvents.dataReceivedFromSource(buffer, sourceSocket);
            const destinationSocket = yield proxyConnection.onData(buffer);
            if (destinationSocket) {
                destinationSocket.write(buffer);
                this.domainEvents.dataSentToDestination(buffer, destinationSocket);
            }
        }));
        sourceSocket.on('close', () => {
            proxyConnection.close();
            this.domainEvents.sourceSocketClosed(sourceSocket);
        });
        sourceSocket.on('error', (error) => {
            if (!error) {
                return;
            }
            proxyConnection.close();
            this.domainEvents.sourceSocketError(error, sourceSocket);
        });
    }
}
exports.ProxyServer = ProxyServer;
//# sourceMappingURL=proxy-server.js.map