"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const tls = require("tls");
class TLSServerBuilder {
    constructor() {
        this.certificate = null;
        this.connectionFn = null;
        this.key = null;
    }
    build() {
        return tls.createServer({
            cert: this.certificate,
            key: this.key,
            rejectUnauthorized: false,
        }, (socket) => this.connectionFn(socket));
    }
    reset() {
        this.connectionFn = null;
        return this;
    }
    setCertificateFromPath(filePath) {
        this.certificate = fs.readFileSync(filePath);
        return this;
    }
    setConnectionFn(connectionFn) {
        this.connectionFn = connectionFn;
        return this;
    }
    setKeyFromPath(filePath) {
        this.key = fs.readFileSync(filePath);
        return this;
    }
}
exports.TLSServerBuilder = TLSServerBuilder;
//# sourceMappingURL=tls.js.map