"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const socks5_1 = require("./socks5");
const socks5_command_1 = require("./socks5-command");
const socks5_address_type_1 = require("./socks5-address-type");
const socks5_status_1 = require("./socks5-status");
const dns_1 = require("../helpers/dns");
const hexadecimal_1 = require("../helpers/hexadecimal");
class SOCKS5ConnectionRequest {
    constructor(version, command, addressType, address, addressBytes, port, portBytes) {
        this.version = version;
        this.command = command;
        this.addressType = addressType;
        this.address = address;
        this.addressBytes = addressBytes;
        this.port = port;
        this.portBytes = portBytes;
    }
    static fromBuffer(buffer) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = new SOCKS5ConnectionRequest(buffer[0], buffer[1], buffer[3], null, null, null, null);
            if (result.version !== socks5_1.SOCKS5Constants.VERSION) {
                return null;
            }
            if (result.addressType === socks5_address_type_1.SOCKS5AddressType.DOMAIN_NAME) {
                const domainNameLength = buffer[4];
                const domainName = buffer.slice(5, 5 + domainNameLength).toString();
                try {
                    result.address = yield dns_1.DNSHelper.resolveDomainName(domainName);
                }
                catch (_a) {
                    return {
                        result,
                        status: socks5_status_1.SOCKS5Status.HOST_UNREACHABLE,
                    };
                }
                result.portBytes = [buffer[5 + domainNameLength], buffer[5 + domainNameLength + 1]];
                result.port = hexadecimal_1.HexadecimalHelper.toDecimal(result.portBytes);
            }
            else if (result.addressType === socks5_address_type_1.SOCKS5AddressType.IPv4) {
                result.addressBytes = [buffer[4], buffer[5], buffer[6], buffer[7]];
                result.address = `${result.addressBytes[0]}.${result.addressBytes[1]}.${result.addressBytes[2]}.${result.addressBytes[3]}`;
                result.portBytes = [buffer[8], buffer[9]];
                result.port = hexadecimal_1.HexadecimalHelper.toDecimal(result.portBytes);
            }
            else if (result.addressType === socks5_address_type_1.SOCKS5AddressType.IPv6) {
                return {
                    result,
                    status: socks5_status_1.SOCKS5Status.ADDRESS_TYPE_NOT_SUPPORTED,
                };
            }
            switch (result.command) {
                case socks5_command_1.SOCKS5Command.TCPIP_PORT_CONNECTION:
                    return {
                        result,
                        status: socks5_status_1.SOCKS5Status.COMMAND_NOT_SUPPORTED,
                    };
                case socks5_command_1.SOCKS5Command.TCPIP_STREAM_CONNECTION:
                    return {
                        result,
                        status: socks5_status_1.SOCKS5Status.GRANTED,
                    };
                case socks5_command_1.SOCKS5Command.UDP_PORT:
                    return {
                        result,
                        status: socks5_status_1.SOCKS5Status.COMMAND_NOT_SUPPORTED,
                    };
                default:
                    return {
                        result,
                        status: socks5_status_1.SOCKS5Status.COMMAND_NOT_SUPPORTED,
                    };
            }
        });
    }
}
exports.SOCKS5ConnectionRequest = SOCKS5ConnectionRequest;
//# sourceMappingURL=socks5-connection-request.js.map